'use client';
import { enrichEventWithDetails, ThemingParameters, useStylesheet } from '@ui5/webcomponents-react-base';
import { clsx } from 'clsx';
import React, { createElement, forwardRef, useCallback, useMemo } from 'react';
import { getValueByDataKey } from 'recharts/lib/util/ChartUtils.js';
import { ChartContainer } from '../../internal/ChartContainer.js';
import { defaultFormatter } from '../../internal/defaults.js';
import { BarChartPlaceholder } from '../BarChart/Placeholder.js';
import { classNames, styleData } from './MicroBarChart.module.css.js';
const resolveColor = (index, color = null) => {
    if (color) {
        return ThemingParameters[color] ?? color;
    }
    return ThemingParameters[`sapChart_Sequence_${(index % 11) + 1}`];
};
/**
 * The `MicroBarChart` compares different values of the same category to each other by displaying them in a compact way.
 */
const MicroBarChart = forwardRef((props, ref) => {
    const { loading, dataset, onDataPointClick, style, className, slot, ChartPlaceholder, ...rest } = props;
    useStylesheet(styleData, MicroBarChart.displayName);
    const dimension = useMemo(() => ({
        formatter: defaultFormatter,
        ...props.dimension
    }), [props.dimension]);
    const measure = useMemo(() => ({
        formatter: defaultFormatter,
        ...props.measure
    }), [props.measure]);
    const maxValue = useMemo(() => {
        if (dataset) {
            const maxDatasetValue = Math.max(...dataset?.map((item) => getValueByDataKey(item, measure.accessor)));
            return props.maxValue ?? maxDatasetValue;
        }
        return 0;
    }, [dataset, measure, props.maxValue]);
    const barHeight = measure?.width ? `${measure.width}px` : '4px';
    const onBarClick = useCallback((item, index) => (e) => {
        if (typeof onDataPointClick === 'function') {
            onDataPointClick(enrichEventWithDetails(e, {
                dataKey: measure.accessor,
                value: getValueByDataKey(item, measure.accessor),
                payload: item,
                dataIndex: index
            }));
        }
    }, [measure.accessor, onDataPointClick]);
    const barContainerClasses = clsx(classNames.barContainer, onDataPointClick && classNames.barContainerActive);
    const { maxValue: _0, dimension: _1, measure: _2, ...propsWithoutOmitted } = rest;
    return (React.createElement(ChartContainer, { dataset: dataset, loading: loading, Placeholder: ChartPlaceholder ?? BarChartPlaceholder, ref: ref, style: style, className: className, slot: slot, resizeDebounce: 250, ...propsWithoutOmitted },
        React.createElement("div", { className: classNames.container }, dataset?.map((item, index) => {
            const dimensionValue = getValueByDataKey(item, dimension.accessor);
            const measureValue = getValueByDataKey(item, measure.accessor);
            const formattedDimension = dimension.formatter(dimensionValue);
            let formattedMeasure = '';
            if (!measure.hideDataLabel) {
                if (measure.DataLabel) {
                    formattedMeasure = createElement(measure.DataLabel, {
                        value: measureValue,
                        config: measure,
                        formattedValue: measure.formatter(measureValue)
                    });
                }
                else {
                    formattedMeasure = measure.formatter(measureValue);
                }
            }
            return (React.createElement("div", { key: dimensionValue, className: barContainerClasses, onClick: onBarClick(item, index) },
                React.createElement("div", { className: classNames.labelContainer },
                    React.createElement("span", { className: classNames.label, title: formattedDimension }, formattedDimension),
                    React.createElement("span", { className: classNames.text, title: formattedMeasure }, measure.hideDataLabel ? '' : formattedMeasure)),
                React.createElement("div", { className: classNames.valueContainer, style: {
                        opacity: measure?.opacity ?? 1,
                        height: barHeight
                    } },
                    React.createElement("div", { className: classNames.valueBar, "data-component-name": "MicroBarChartValueBar", style: {
                            width: `${(measureValue / maxValue) * 100}%`,
                            backgroundColor: resolveColor(index, measure?.colors?.[index])
                        } }))));
        }))));
});
MicroBarChart.displayName = 'MicroBarChart';
export { MicroBarChart };
